;********************************************************************************************
;
;	ROUTINES PER EFFETTUARE IL CONTROLLO DI SERVIMOTORE DA MODELLISMO
;	(HS-81 Microservo) SPECIFICHE PER "PIC18FXXX"
;	Versione 2.0		     		Ult.agg. 26/07/03
;
;********************************************************************************************

;********************************************************************************************
;
;	NOTA: PER FUNZIONARE LA ROUTINE "SERVO_UPDATE" HA BISGNO DI ESSERE 
;	      CHIAMATA OGNI 20 ms!(Interrup sul modulo CCP timer1)
;	      SONO RICHIESTI INOLTRE L' INCLUDE DEL FILE "f18macro.asm"
;				
;********************************************************************************************

;--- REGISTRI DA DEFINIRE NEL MAIN PROGRAM!  -------------------;
;	FRAME, SERV_TEMP					;
;	VALUE_SERV_0, VALUE_SERV_1, VALUE_SERV_2, VALUE_SERV_3	;
;	VALUE_SERV_4, VALUE_SERV_5, VALUE_SERV_6, VALUE_SERV_7	;

;-------------------------------;
; PARAMETRI IMPOSTABILI		;
;-------------------------------;
#DEFINE NUM_OF_SERV	D'8'	; I SERVI DEVONO ESSERE MESSI SU PIN
				; CONSECUTIVI A PARTIRE DA 0. 
#DEFINE	PORT_SERV	PORTB	; DEFINIZIONE PORTA PER I SERVI
#DEFINE	TRIS_SERV	TRISB	;


	variable MASK_SERV

	IF NUM_OF_SERV == 0
	ERROR	"Selezionare almeno un servomotore!"
	ENDIF

	IF NUM_OF_SERV == 1
		MASK_SERV = B'00000001'
	ENDIF
	IF NUM_OF_SERV == 2
		MASK_SERV = B'00000011'
	ENDIF
	IF NUM_OF_SERV == 3
		MASK_SERV = B'00000111'
	ENDIF
	IF NUM_OF_SERV == 4
		MASK_SERV = B'00001111'
	ENDIF
	IF NUM_OF_SERV == 5
		MASK_SERV = B'00011111'
	ENDIF
	IF NUM_OF_SERV == 6
		MASK_SERV = B'00111111'
	ENDIF
	IF NUM_OF_SERV == 7
		MASK_SERV = B'01111111'
	ENDIF
	IF NUM_OF_SERV == 8
		MASK_SERV = B'11111111'
	ENDIF
	IF NUM_OF_SERV > 8
	ERROR	"Troppi Servomotori!"
	ENDIF


SERVO_INIT
	MOVLW	MASK_SERV	; SELEZIONO I PIN DEI SERVOMOTORI COME OUTPUT
	COMF	WREG, W		;
	ANDWF	TRIS_SERV, F	;
	ANDWF	PORT_SERV, F	;
	RETURN

	
;-----------------------------------------------;
;						;
;	MACRO NECESSARIE A SERVLOOP		;
;						;
;-----------------------------------------------;
;	(2 * NUM_OF_SERV) Istruzioni		;
;-----------------------------------------------;
N_Servi	macro					;
	local	a = 0				;
	while	a < NUM_OF_SERV			;
						;
	CPFSGT	VALUE_SERV_#v(a)		; 
	BCF	SERV_TEMP, #v(a)		; 
						;
a ++						;
	endw					;
	endm					;
;-------------------------------------------------------------------------------;
;	Il numero di istruzioni in 5 us :					;
;	D'5' * FREQUENCY / D'4000'						;
;	Dunque il numero di nop da inserire :					;
;	(D'5' * FREQUENCY / D'4000') - (D'6' + D'2' * NUM_OF_SERV)		;
;-------------------------------------------------------------------------------;
N_Nop	macro									;
	variable NomNumFix							;
	NomNumFix = (D'5' * FREQUENCY / D'4000') - (D'6' + D'2' * NUM_OF_SERV)	;
	if	(D'5' * FREQUENCY / D'4000') < (D'6' + D'2' * NUM_OF_SERV)		;
	error "La frequenza  troppo bassa per pilotare tutti i servimotore!"		;
	endif										;
	if	((D'5' * FREQUENCY) % D'4000') != 0					;
	messg "Il controllo dei servi NON sar esatto, causa frequenza sistema"		;
	endif										;
	local	nopnum = 0							;
	while	nopnum < NomNumFix						;
	NOP									;
	nopnum ++								;
	endw									;
	endm									;
;-------------------------------------------------------------------------------;


SERVO_UPDATE
	CLRF	FRAME		; ALZO SOLO I PIN DEI SERVI ATTIVI
	SETF	SERV_TEMP	;
	MOVLW	MASK_SERV	; COSI' AGISCO SOLO SUI PIN EFFETTIVAMENTE
	IORWF	PORT_SERV, F	; COLLEGATI AD UN SERVO!
	
		
	DelayMicroSec	D'860'	; 0.860 ms DI ATTESA (per utilizzare tutti i bit ... 
				; ... vado leggermente fuori le specifiche del servo)

SERVLOOP			; IL LOOP, ESCLUSI I NOP, E' DI (2 * NUM_OF_SERV + 6) ISTRUZIONI
	MOVF	FRAME, W	; E DEVE DURARE ESATTAMENTE 5 us

	N_Servi			; La macro occupa (2 * NUM_OF_SERV) Istruzioni.
	N_Nop			; Sono aggiunti tanti NOP quanti ne servono perch 
				; la routine SERVLOOP duri esattamente 5 us!
				
				;-----------------------------------------------
	MOVF	SERV_TEMP, W	; COSI' AGISCO SOLO SUI PIN EFFETTIVAMENTE	
	ANDWF	PORT_SERV, F	; COLLEGATI AD UN SERVO!
	INCFSZ	FRAME, F	; 
	GOTO	SERVLOOP	; QUANDO HO FATTO TUTTI I FRAME POSSO USCIRE	
	RETURN			;-----------------------------------------------
