;***************************************************************************************;
;											;
;	Firmware for Servo Motor Control						;
;											;
;	Author:	VIRGILIO MATTOLI		JULY-08-2004 (Version 1.0)		;
;	Frequency = 16 MHz di clock (4MHz x 4PLL)					;
;											;
;***************************************************************************************;


	LIST P = 18F252			; 
	include <P18F252.inc>		; 
					; La routine e' per i 16 MHz di clock (4MHz x 4PLL)--> 4MP/s
	#define FREQUENCY D'40000'	; Frequenza del procesore in KHz (va considerato anche l'eventuale PLL)

        include "18fXX2_conf.h"	;	

	CBLOCK 0x15
        TEMP, TEMP1, TEMP2, TEMP3 

	TIME_STEP_H, TIME_STEP_L
	TOT_NUM_STEP_H, TOT_NUM_STEP_L, PAGE_COUNTER	; TOT_NUM_STEP == NUM OF BYTE!!!
	MAX_STEP_H, MAX_STEP_L				; MAX_STEP == NUM OF STEP!!!
	STEP_COUNTER_H, STEP_COUNTER_L
	MOV, XNUM_SPEED, XSPEED	
	CINQUANTESIMI, QUARTI, SECONDI, MINUTI

	FRAME, SERV_TEMP					
	VALUE_SERV_0, VALUE_SERV_1, VALUE_SERV_2, VALUE_SERV_3	
	VALUE_SERV_4, VALUE_SERV_5, VALUE_SERV_6, VALUE_SERV_7	
	
	KEY, TEMP_KEY, TEMP_KEY2, TEMP_KEY3

	BYTE_NUM, FLASH_COUNTER
	FLASH_ADDR_U, FLASH_ADDR_H, FLASH_ADDR_L
	FLASH_BUFF

	ENDC


;       I/O ports
;------------------------------------------------------------
;       Serial comms.   :       PORTC <7:6>
;	Servo Output	:	PORTB <7:0>
;       Keys            :       PORTC <3:2>
;	LEDs		:	PORTC <5:4>
;------------------------------------------------------------



LED_GREEN	=	4
LED_RED		=	5
PORT_LED	=	PORTC
TRIS_LED	=	TRISC
#DEFINE	G_LED	PORT_LED, LED_GREEN
#DEFINE	R_LED	PORT_LED, LED_RED

#DEFINE	NUM_OF_SERVO_CHANN	D'8'	; NUM_OF_SERVO_CHANN IS DIFFERENT TO NUM_OF_SERVO
#DEFINE	FLASH_INDEX_OFFSET	0x2100
#DEFINE	FLASH_OFFSET		(0x2100 + 0x0008)

#DEFINE	START_SPEED	D'1'	;; (PERIODO DI AGGIORNAMENTO INIZIALE ... X 20 ms)

	ORG	0x00
	GOTO	MAIN

;*********************************************************************
;
;	ROUTINE DI INTERRUPT
;

	ORG     0x08                    	; ROUTINE DI INTERRUPT
        GOTO    INTERRUPT       	;
;*****************************************************************
;*****************************************************************


;*****************************************************************
;*****************************************************************
;
; QUI CI SONO I FILE DI INCLUDE E DI MACRO!
;

	include "18fmacro.asm"		; Va definito sempre per primo
	include "18f_perf.asm"		;	
	include "18_Math.asm"		;
	include "ServoRoutines.asm"	;	
	include "ServoF18.asm"		;	
	include "18f_flash.asm"		;
	include "Keys18F.asm"		;
	
;
;*****************************************************************
;*****************************************************************



	

;***************************************************************
;
;	OPERAZIONI DA COMPIERE ALL'INTERRUPT
;
;***************************************************************

INTERRUPT:
	BTFSC	PIR1, RCIF		;
	CALL	DECODE			;
	
	BTFSC	PIR2, CCP2IF		;
	CALL	TIMER_UPDATE		;

        RETFIE	FAST			;




;****************************************************************************
;****************************************************************************
;
; TICK DI SISTEMA (Scatta ogni 50 ms)
;
;****************************************************************************
;****************************************************************************

TIMER_UPDATE:
	BCF	PIR2, CCP2IF		; CLEAR FLAG of COMPARE INTERRUPT 

	INCF	CINQUANTESIMI, F	; 1/50 sec!

	CALL	SERVO_UPDATE		;

	BTFSS	MOV, 0			;
	BRA	END_UPDATE		;
	
	MOVLW	0x00			; DECREMENT TIME_STEP
	DECF	TIME_STEP_L, F		;
	SUBWFB	TIME_STEP_H, F		;
	MOVF	TIME_STEP_H, W		; TEST IF ZERO
	IORWF	TIME_STEP_L, W		; SE <> 0 THEN SKIP TO END_UPDATE
	BNZ	END_UPDATE		; (NOTHING TO UPDATE)

	INFSNZ	STEP_COUNTER_L		; STEP COUNTER INCREMENT!
	INCF	STEP_COUNTER_H		;	

	MOVF	MAX_STEP_L, W		; IF STEP COUNTER == MAX_STEP
	SUBWF	STEP_COUNTER_L, W	; 
	BNZ	VALUE_UPDATE		; 

	MOVF	MAX_STEP_H, W		;
	SUBWF	STEP_COUNTER_H, W	; 
	BNZ	VALUE_UPDATE		; THEN

	CLRF	STEP_COUNTER_H		; STEP COUNTER RESET!
	CLRF	STEP_COUNTER_L		;
	TabSetting	FLASH_OFFSET	;

VALUE_UPDATE:
	CALL	READ_STEP_FROM_FLASH	;
		
END_UPDATE:
	MOVLW	D'50'			;
	CPFSEQ	CINQUANTESIMI		; SE CINQUANTESIMI = 50 ALLORA SKIP IL RETURN
	RETURN				; E INCREMENTO I SECONDI
	CLRF	CINQUANTESIMI		;
	RETURN				; 
	
;****************************************************************************
;****************************************************************************

MAIN:
	CALL	SETPORTS		;
	CALL	VAR_INIT		;

HALLO:      
	BSF	G_LED			;
	MILLISECONDI	D'500'		;
	BCF	G_LED			;
	BSF	R_LED			;
	MILLISECONDI	D'500'		;
	BSF	G_LED			;
	BCF	R_LED			;
	MILLISECONDI	D'500'		;
	BCF	G_LED			;

					;; ABILITAZIONE COMUNICAZIONE SERIALE
	BSF     RCSTA,SPEN      	;; RX/TX MODULE ACTIVED
	CLRF    RCREG			;;
	BSF	PIE1, RCIE		;; RX INTERRUPT ON
	BSF	INTCON, GIE		;

MAINLOOP:				;
	CALL	WAITFORKEY		;
	GOTO 	MAINLOOP		;

KEY_2_Routine					;
	BTFSS	MOV, 0				;
	GOTO	TIMER_MOV_ON			;
	RETURN					;	

KEY_2_PRESS_Routine				;
	RETURN					;	

KEY_3_Routine					;
	BTFSC	MOV, 0				;
	GOTO	TIMER_MOV_OFF			;
	RETURN					;

KEY_3_PRESS_Routine				; AUTO-ZERO SEQUENCE
	BTFSC	MOV, 0				;
	RETURN					;	
	TabSetting	FLASH_OFFSET		;
	CALL	READ_STEP_FROM_FLASH		;
	CALL	TIMER_ON			;
	BSF	R_LED				;
	MILLISECONDI	D'1000'			;
	BCF	R_LED				;
	CALL	TIMER_OFF			;
	RETURN					;	

;-----------------------------------------------------


TIMER_ON:
	BSF	G_LED			;
	CLRF	TMR3L			;
	CLRF	TMR3L			;
	BCF	PIR2, CCP2IF		;
	BSF	PIE2, CCP2IE		; SETTING INTERRUPT
	BSF	T3CON, TMR3ON		; START TIMER3
	RETURN

TIMER_OFF:
	BCF	G_LED			;
	BCF	PIR2, CCP2IF		;
	BCF	PIE2, CCP2IE		; RESETTING INTERRUPT
	BCF	T3CON, TMR3ON		; START TIMER3
	BCF	PIR2, CCP2IF		;
	RETURN


TIMER_MOV_ON:
	
	CALL	TIMER_ON		;
	CLRF	STEP_COUNTER_H		;
	CLRF	STEP_COUNTER_L		;
	TabSetting	FLASH_OFFSET	;
	CALL	READ_STEP_FROM_FLASH	;
	BSF	R_LED			;
	SETF	MOV			; "MOV" = 1
	RETURN				;

TIMER_MOV_OFF:
	CALL	TIMER_OFF		;
	BCF	R_LED			;
	CLRF	MOV			; "MOV" = 0
	RETURN




	ORG	FLASH_INDEX_OFFSET
	DB	D'0',	D'100'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'


	ORG	FLASH_OFFSET
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'10',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'20',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'30',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'40',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'50',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'60',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'70',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'80',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'90',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'30'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'20'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'
	DB	D'0',	D'0'



	END

 
