;********************************************************************************************
;
;	ROUTINES
;
;********************************************************************************************

;********************************************************************************************
;
;	- SETPORTS
;	- VAR_INIT
;	- DECODE
;
;********************************************************************************************

;****************************************************************************
;****************************************************************************
;****************************************************************************

SETPORTS
	
	CLRF	INTCON		;;
        movlw   B'00000110'	;;
        movwf   ADCON1  	;; digital I/O PORTA

        movlw   B'11111111'	;;
        movwf   TRISA   	;; PORTA
        movlw   B'11111111'	;;
        movwf   TRISB   	;; PORTB
        movlw   B'11111111'	;;
        movwf   TRISC   	;; PORTC

        BSF	INTCON2, 7	;; DISABLE pull-ups 
	
	BCF	TRIS_LED, LED_RED
	BCF	TRIS_LED, LED_GREEN

	BCF	R_LED		;;
	BCF	G_LED		;;

	CALL	SERVO_INIT	;; SETTING OUT SERVI
	
        CLRF	PIE1            ;; interrupt periferici
        CLRF	PIE2            ;;

	DEV_SETTING		;;
	CALL INIT_KEYS		;;
	RETURN			;;


;****************************************************************************
VAR_INIT
	CLRF	CINQUANTESIMI	;
	CLRF	QUARTI		;
	CLRF	SECONDI		;
	CLRF	MINUTI		;

	MOVLW	START_SPEED	;
	MOVWF	XSPEED		;
	MOVWF	XNUM_SPEED	;
	CLRF	MOV		;
	

	TabSetting FLASH_INDEX_OFFSET
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_H			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_L			;
	TabSetting	FLASH_OFFSET		;

RES_VAL	MOVLW	0x80		; SERVI CENTRATI
	MOVWF	VALUE_SERV_0	;
	MOVWF	VALUE_SERV_1	;
	MOVWF	VALUE_SERV_2	;
	MOVWF	VALUE_SERV_3	;
	MOVWF	VALUE_SERV_4	;
	MOVWF	VALUE_SERV_5	;
	MOVWF	VALUE_SERV_6	;
	MOVWF	VALUE_SERV_7	;

	CLRF	STEP_COUNTER_L		;
	CLRF	STEP_COUNTER_H		;
	MOVLW	D'0'			;
	MOVWF	BARGB0			;
	MOVLW	D'10'			;
	MOVWF	BARGB1			;
	MOVFF	TOT_NUM_STEP_H, AARGB0	;		
	MOVFF	TOT_NUM_STEP_L, AARGB1	;		
	CALL	FXD1616U		;
	MOVFF	AARGB0, MAX_STEP_H	;
	MOVFF	AARGB1, MAX_STEP_L	;
	TabSetting FLASH_OFFSET		;
	RETURN



;****************************************************************************
;****************************************************************************
;****************************************************************************




;***************************************************************
;
;	 CODIFICA COMUNICAZIONE 
;
;***************************************************************

_STOP	=	B'00000000'		;
_START	=	B'00000001'		;
_LOAD = 	B'00000010'		; UPDATE MEMORY
_START_MOV =	B'00000011'		;
_STOP_MOV =	B'00000100'		;
_SPEED_MOV =	B'00000101'		; UPDATE SPEED
_DOWNLOAD =	B'00000110'		; DOWNLOAD MEMORY
_ERASE =	B'00000111'		; ERASE MEMORY
_TEST =		B'00010000'		; ERASE MEMORY
_RESET =	B'00010001'		; RESET PROCESSOR

_S_0	=	B'00001000'		; UPDATE SERVO 0
_S_1	=	B'00001001'		; UPDATE SERVO 1
_S_2	=	B'00001010'		; UPDATE SERVO 2
_S_3	=	B'00001011'		; UPDATE SERVO 3
_S_4	=	B'00001100'		; UPDATE SERVO 4
_S_5	=	B'00001101'		; UPDATE SERVO 5
_S_6	=	B'00001110'		; UPDATE SERVO 6
_S_7	=	B'00001111'		; UPDATE SERVO 7


;***************************************************************
;
;	DECODIFICA ISTRUZIONI SERIALI
;
;***************************************************************

DECODE
        MOVFF	RCREG, TEMP     ;
;	MOVFF	TEMP, TXREG	; ECHO	

        BCF     STATUS, Z       ;
        MOVLW   _START		; Start Servo Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	START		;

        BCF     STATUS, Z       ;
        MOVLW   _START_MOV	; Start MOV Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	START_MOV	;
        
        BCF     STATUS, Z       ;
        MOVLW   _STOP		; Stop Servo Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO 	STOP		;

        BCF     STATUS, Z       ;
        MOVLW   _STOP_MOV	; Stop MOV Update
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO 	STOP_MOV	;

        BCF     STATUS, Z       ;
        MOVLW   _SPEED_MOV	; UPDATE via PC SPEED
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SPEED_MOV	;

        BCF     STATUS, Z       ;
        MOVLW   _S_0		; UPDATE via PC SERVO_0
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_0		;

        BCF     STATUS, Z       ;
        MOVLW   _S_1		; UPDATE via PC SERVO_1
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_1		;

        BCF     STATUS, Z       ;
        MOVLW   _S_2		; UPDATE via PC SERVO_2
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_2		;

        BCF     STATUS, Z       ;
        MOVLW   _S_3		; UPDATE via PC SERVO_3
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_3		;

        BCF     STATUS, Z       ;
        MOVLW   _S_4		; UPDATE via PC SERVO_4
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_4		;

        BCF     STATUS, Z       ;
        MOVLW   _S_5		; UPDATE via PC SERVO_5
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_5		;
				;
        BCF     STATUS, Z       ;
        MOVLW   _S_6		; UPDATE via PC SERVO_6
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_6		;

        BCF     STATUS, Z       ;
        MOVLW   _S_7		; UPDATE via PC SERVO_6
        SUBWF   TEMP, W         ;
        BTFSC   STATUS, Z       ;
        GOTO	SERVO_7		;
				;
	BCF	STATUS, Z	; LOAD PROGRAM
	MOVLW	_LOAD		; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	LOAD_PROG	; 
				;
	BCF	STATUS, Z	; DOWNLOAD PROGRAM
	MOVLW	_DOWNLOAD	; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	SEND_PROG	; 
				;
	BCF	STATUS, Z	; ERASE PROGRAM
	MOVLW	_ERASE		; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	ERASE_ZERO	; 

	BCF	STATUS, Z	; DOWNLOAD PROGRAM
	MOVLW	_TEST		; 
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	GOTO	TEST_PROG	; 

	BCF	STATUS, Z	;
	MOVLW	_RESET		; RESET PROCESSOR
	SUBWF	TEMP, W		;
	BTFSC	STATUS, Z	;
	RESET			; 

	RETURN			;

;*******************************;

;	BCF	STATUS, Z	;
;	MOVLW	.50		; ...
;	SUBWF	TEMP, W		;
;	BTFSC	STATUS, Z	;
;	CALL	....		; 


SPEED_MOV	
;	BTFSS	PIR1, RCIF		;
;	BRA	SPEED_MOV		; ATTENDO IL DUTY PER IL PWM
;	MOVFF	RCREG, SPEED		; METTO IL VALORE DELLA VELOCITA' NEL REGISTRO GIUSTO
	RETURN


SERVO_0	BTFSS	PIR1, RCIF		;
	BRA	SERVO_0			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_0	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_1	BTFSS	PIR1, RCIF		;
	BRA	SERVO_1			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_1	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_2	BTFSS	PIR1, RCIF		;
	BRA	SERVO_2			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_2	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_3	BTFSS	PIR1, RCIF		;
	BRA	SERVO_3			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_3	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_4	BTFSS	PIR1, RCIF		;
	BRA	SERVO_4			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_4	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_5	BTFSS	PIR1, RCIF		;
	BRA	SERVO_5			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_5	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_6	BTFSS	PIR1, RCIF		;
	BRA	SERVO_6			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_6	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN

SERVO_7	BTFSS	PIR1, RCIF		;
	BRA	SERVO_7			; ATTENDO IL DUTY PER IL PWM
	MOVFF	RCREG, VALUE_SERV_7	; METTO IL VALORE DEL DUTY NEL REGISTRO GIUSTO
	RETURN



START
	CALL	TIMER_ON
	RETURN

START_MOV
	CALL	TIMER_MOV_ON
	RETURN

STOP
	CALL	TIMER_OFF
	RETURN

STOP_MOV
	CALL	TIMER_MOV_OFF
	RETURN


TEST_PROG
	BSF	R_LED

	MOVLW	upper FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_U			;
	MOVLW	high FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_H			;
	MOVLW	low FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_L			;

	MOVLW	D'1'				;
	MOVWF	BYTE_NUM			;
	CALL	ERASE_FLASH			;

	MOVLW	upper FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_U			;
	MOVLW	high FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_H			;
	MOVLW	low FLASH_OFFSET		;
	MOVWF	FLASH_ADDR_L			;

	MOVLW	D'130'				;
	LFSR	0, FLASH_BUFF			; SETTING RAM POINTER
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;
	MOVWF	POSTINC0			;

	MOVLW	D'8'				;
	MOVWF	BYTE_NUM			;

	CALL	WRITE_FLASH			; WHIT ADDRESSING
	BCF	R_LED
	CALL	RES_VAL				;
	TabSetting	FLASH_OFFSET		;
	RETURN


LOAD_PROG:
	BSF	R_LED
	BTFSS	PIR1, RCIF		;
	BRA	$-2			; ATTENDO IL NUMERO DI BYTE
	MOVFF	RCREG, TOT_NUM_STEP_H	;
	BTFSS	PIR1, RCIF		;
	BRA	$-2			; ATTENDO IL NUMERO DI BYTE
	MOVFF	RCREG, TOT_NUM_STEP_L	;

	CALL	ERASE

	BSF	R_LED
	MOVLW	upper FLASH_INDEX_OFFSET	; Pointer to TOT_NUM_OF_STEP
	MOVWF	FLASH_ADDR_U			;
	MOVLW	high FLASH_INDEX_OFFSET		;
	MOVWF	FLASH_ADDR_H			;
	MOVLW	low FLASH_INDEX_OFFSET		;
	MOVWF	FLASH_ADDR_L			;
		
	MOVFF	TOT_NUM_STEP_H, FLASH_BUFF	; To Recover After
	MOVFF	TOT_NUM_STEP_L,(FLASH_BUFF +1 )	;

	MOVLW	D'8'				;
	MOVWF	BYTE_NUM			;

	CALL	WRITE_FLASH			; WHIT ADDRESSING

	MOVLW	0xFF				; ACK
	MOVWF	TXREG				;

	TabSetting (FLASH_OFFSET - D'8')	;  OFFESET ADJUST! (FOR BUFFER LOADING SHIFT)
	
PAGE_LOOP:
	LFSR	0, FLASH_BUFF			; SETTING RAM POINTER
	
	MOVFF	TOT_NUM_STEP_L, PAGE_COUNTER	;
	MOVF	TOT_NUM_STEP_H, F		; IF TOT_NUM_STEP_H = 0 ...
	BZ	RX_PAGE_LOOP			; ... THEN PAGE_COUNTER == 256
	CLRF	PAGE_COUNTER			; ELSE PAGE_COUNTER == TOT_NUM_STEP_L

RX_PAGE_LOOP:
	BTFSS	PIR1, RCIF		;
	BRA	$-2			; ATTENDO IL NUMERO DI BYTE
	MOVF	RCREG, W		;
	MOVWF	POSTINC0		; IN RAM WHIT POST INCREMENT
	DECFSZ	PAGE_COUNTER		; 256 CICLI!
	BRA	RX_PAGE_LOOP		; PAGE_COUNTER
	
	LFSR	0, FLASH_BUFF		; SETTING RAM POINTER
	CLRF	BYTE_NUM		; SETTING NUM OF BYTE TO WRITE (0 = 256)
	CALL	WRITE_FLASH_NO_ADDR_NO_OFFSET

	MOVLW	0xFF			; ACK
	MOVWF	TXREG			;

	MOVF	TOT_NUM_STEP_H, F	;
	BZ	EXIT_LOOP		;
	DECF	TOT_NUM_STEP_H, F	;
	BRA	PAGE_LOOP		;
EXIT_LOOP:
	TabSetting FLASH_INDEX_OFFSET
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_H			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_L			;
	TabSetting	FLASH_OFFSET		;
	CALL	RES_VAL
	BCF	R_LED				;
	RETURN					;


SEND_PROG:
	BSF	R_LED
	TabSetting FLASH_INDEX_OFFSET		;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	ANDLW	B'00111111'			; SAFE FOR AVOID RS232 BUFFER OVERFLOW!
	MOVWF	TOT_NUM_STEP_H			;
	CALL	RS232PUTCHAR			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_L			;
	CALL	RS232PUTCHAR			;
	DelayMilliSec	D'10'			;
	
	MOVF	TOT_NUM_STEP_H, W		; TEST IF ZERO
	IORWF	TOT_NUM_STEP_L, W		;
	BZ	EXIT_SEND_LOOP			;
	
	TabSetting	FLASH_OFFSET		;
SEND_LOOP:					;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	CALL	RS232PUTCHAR			;

	DECF	TOT_NUM_STEP_L, F		; DECREMENT 2 BYTE NUMBER
	MOVLW	0x00				;
	SUBWFB	TOT_NUM_STEP_H, F		;
	
	MOVF	TOT_NUM_STEP_H, W		; TEST IF ZERO
	IORWF	TOT_NUM_STEP_L, W		;
	BZ	EXIT_SEND_LOOP			;

	BRA	SEND_LOOP

EXIT_SEND_LOOP:
	TabSetting FLASH_INDEX_OFFSET		;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_H			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TOT_NUM_STEP_L			;
	TabSetting	FLASH_OFFSET		;
	CALL	RES_VAL
	BCF	R_LED
	RETURN



ERASE:
	TabSetting FLASH_INDEX_OFFSET				;
	MOVLW	((((NUM_OF_SERVO_CHANN + 2)*1024)/64) +1)	;
	MOVWF	BYTE_NUM					;
	BSF	R_LED						;
	CALL	ERASE_FLASH_NO_ADDR				;
	BCF	R_LED						;
	RETURN

ERASE_ZERO:
	BSF	R_LED				;

	CALL	ERASE				;

	MOVLW	upper FLASH_INDEX_OFFSET	;
	MOVWF	FLASH_ADDR_U			;
	MOVLW	high FLASH_INDEX_OFFSET		;
	MOVWF	FLASH_ADDR_H			;
	MOVLW	low FLASH_INDEX_OFFSET		;
	MOVWF	FLASH_ADDR_L			;

	MOVLW	D'0'			;
	LFSR	0, FLASH_BUFF		; ZERO STEPS
	MOVWF	POSTINC0		;
	MOVWF	POSTINC0		;

	MOVLW	D'8'			;
	MOVWF	BYTE_NUM		;

	CALL	WRITE_FLASH		; WHIT ADDRESSING
	TabSetting	FLASH_OFFSET	;
	BCF	R_LED			;
	RETURN				;



RS232PUTCHAR:
	BTFSS	PIR1, TXIF		;
	BRA	$ - 2			;
	MOVWF	TXREG			;
	RETURN				;


;****************************************************************************
;****************************************************************************

DELAY_1_SECONDO
	DelayMilliSec	.1000	;; 1 secondo di attesa
	RETURN

;****************************************************************************
;****************************************************************************

F_SERV	MACRO					;
	LOCAL	a = 0				;
	WHILE	a < NUM_OF_SERVO_CHANN		;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	VALUE_SERV_#v(a)		;
	a ++					;
	ENDW					;
	ENDM


READ_STEP_FROM_FLASH
	F_SERV
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TIME_STEP_H			;
	TBLRD*+					;
	MOVF	TABLAT, W			;
	MOVWF	TIME_STEP_L			;
	RETURN
	
	
